/***********************************************************
 * File: CountyJSON.cpp
 *
 * Code to analyze a JSON file of US county population data.
 */

#include <iostream>
#include <string>
#include <fstream>
#include "vector.h"
#include "console.h"
#include "Utilities/JSON.h"
using namespace std;

const int kBaseYear = 2020;

int populationIn(JSON data, int year) {
    /* Base case: If the data is an array, then
     * just read off the total.
     */
    if (data.type() == JSON::Type::ARRAY) {
        return data[year - kBaseYear].asInteger();
    }
    /* Recursive case: This is an object. Its
     * children actually have the numbers. Add
     * them all up.
     */
    int total = 0;
    for (JSON key: data) {
        total += populationIn(data[key], year);
    }
    return total;
}

int main() {
    ifstream input("CountyPopulations.json");
    JSON data = JSON::parse(input);

    for (int year = 2020; year <= 2024; year++) {
        cout << "Population in " << year << ": "
             << populationIn(data["California"], year) << endl;
    }


    return 0;
}





