/*************************************************
 * File: PrettyPrinting.cpp
 *
 * Code to print out a somewhat reasonable representation
 * of a multiway tree.
 */

#include <iostream>
#include <string>
#include "vector.h"
#include "simpio.h"
#include "console.h"
using namespace std;

/* Type: Node
 *
 * A node in a multiway tree.
 */
struct Node {
    string value;
    Vector<Node*> children;
};

/* Returns this exact tree:
 *          A
 *         /|\
 *        / | \
 *       B  C  D
 *        // \\
 *      / /   \ \
 *    /  /     \  \
 *   E   F     G   H
 *  / \       / \
 * I   J     K   L
 *               |
 *               M
 */
Node* makeTree() {
    Node* b = new Node{ "B", {} };
    Node* i = new Node{ "I", {} };
    Node* j = new Node{ "J", {} };
    Node* e = new Node{ "E", {i, j} };
    Node* f = new Node{ "F", {} };
    Node* k = new Node{ "K", {} };
    Node* m = new Node{ "M", {} };
    Node* l = new Node{ "L", {m} };
    Node* g = new Node{ "G", {k, l}};
    Node* h = new Node{ "H", {} };
    Node* c = new Node{ "C", {e, f, g, h}};
    Node* d = new Node{ "D", {} };
    Node* a = new Node{ "A", {b, c, d}};
    return a;
}

void freeTree(Node* root) {
    if (root != nullptr) {
        for (Node* child: root->children) {
            freeTree(child);
        }
        delete root;
    }
}

void prettyPrintRec(Node* tree, const string& prefix) {
    /* Base case: Node has no children. */
    if (tree->children.isEmpty()) {
        cout << prefix << tree->value << endl;
    }
    /* Recursive case: Node has some children. So, print
     * out the name, a brace, the children, and a close
     * brace.
     */
    else {
        cout << prefix << tree->value << " {" << endl;

        for (Node* child: tree->children) {
            prettyPrintRec(child, prefix + "     ");
        }

        cout << prefix << "}" << endl;
    }
}

void prettyPrint(Node* tree) {
    /* Q: Why do I only check for null here, rather than
     * having that be the base case of the recursion,
     * especially since that's what we did with BSTs?
     */
    if (tree == nullptr) return;

    prettyPrintRec(tree, "");
}

int main() {
    Node* tree = makeTree();

    prettyPrint(tree);

    freeTree(tree);
    return 0;
}






